function r=fisher(m,R,N,X)

%fisher: computes the wavelet estimate of the Fisher information
%         of the linear wavelet estimator at scale m
%	  with Daubechies minimum phase wavelets
%***********************************************************
%INPUTS:
% m		wavelet scale
% R		radius of pre-estimator 
%      		 f(X_i) = #(X_j : |X_j-X_i|<= R, j=1:n)
%		 If R=[] then R=1.843*min(Std, IQR/1.449)*n^(-1/5) is used
% N,		Daubechies minimum phase wavelet number
%		(only N=1,2,3,4,5 and 7)
% X		Data
%
%OUTPUT:
% r		wavelet Fisher information estimate
%
%USAGE:
% [r]= fisher(m,R,N,X);
%
%NOTES:
% Function called: cfisher.m
% Tabulated coefficients are only for N=1,2,3,4,5 and 7
% Matlab toolbox required: WavBox (copyright Carl Taswell)
%
%REFERENCE:
% Vannucci, M. and Vidakovic, B. (1997).
% Preventing the Dirac Disaster: wavelet based density estimation.
% Journal of the Italian Statistical Society, 6(2), pp. 145-159
%
%Copyright (c) 1995 Marina Vannucci.
%**********************************************************************

m=-m; % phi(2^(-m)x -k)

[rig,checkc]=size(X);
if checkc==1
else
        X=X';
	rig=checkc;
end

if isempty(R)
 
	R=1.843*min(std(X), iqr(X)/1.449)*(length(X)^(-1/5));
else
end

fb=prfbank(fcdomp(N));   % prfbank.m and cascade.m from WavBox toolbox
y=cascade(fb);
y=y(:,1:1);
NN=length(fb);
J=11-floor(log2(NN));
x=[2^(-J):2^(-J):NN-1];

%-----phi(0)==0
   
x=[0 x];
y=[0 y'];

%------range of k and k1

ak=floor(min(X)*(2^(-m)))-NN+1;
bk=ceil(max(X)*(2^(-m)));
k=ak:bk;
if (N > 1) 
 	L1=-NN+2;
 	L2=NN-2;
else
 	L1=-1;
 	L2=1;
end
k1=ak+L1:bk+L2;

%------ycar==matrix    i-th row is phi((2^m)*Xi-k1) for k1

col=length(k1);
ycar=zeros(rig,col);
for j = 1:col     
  	V = (2^(-m))*X'-k1(j);
  	V = V.*(V>= 0 & V<=(NN-1));
  	[I,JJ,V]=find(V); 
  	V = interp1(x,y,V); 
  	car=sparse(JJ,j,V,rig,col);
  	if ~isempty(JJ) 
		ycar(JJ,j)=car(JJ,j); 
	else 
	end
  	ycar=sparse(ycar);
end

%-------------pre-estimator----------------

for i=1:rig
 	count=abs(X-X(i)) <= R;
 	count=1/sqrt(sum(count));
 	ycar(i,:)=ycar(i,:)*count;
end

%-------Fisher Information, coeff r(m,k)-------------

coefr=cfisher(N);
coe=rot90(rot90(coefr));
r=0;
for i=1:length(k) 
 	kl1=find(k1==k(i)-L1);
 	kl2=find(k1==k(i)-L2);
 	if (kl1 <= kl2) car=ycar(:,kl1:kl2);
 		Cr=sum(coefr.*sum(car));
  		newval=(((2^(-3*m/2))/rig)*Cr)^2;
  		r= r+newval;
 	else
  		car=ycar(:,kl2:kl1);
  		car=sum(car);
  		Cr=sum(coe.*car);
  		newval=(((2^(-3*m/2))/rig)*Cr)^2;
  		r= r+newval;
 	end
end
[Crig,Ccol]=size(ycar);
C=0;
for i=1:Ccol
 	C=C+(sum(ycar(:,i)))^2;
end
C=(2^(-m))*C/(rig^2);
r=r/C;







